@@ -1,4 +1,4 @@
-// $Revision: #24 $$Date: 2003/07/15 $$Author: wsnyder $ -*- SystemC -*-
+// $Revision: #25 $$Date: 2003/07/16 $$Author: wsnyder $ -*- SystemC -*-
 //********************************************************************
 //
 // THIS MODULE IS PUBLICLY LICENSED
@@ -61,12 +61,16 @@ using namespace std;
 // Tracing types
 #define SP_TRACED	// Just a NOP; it simply marks a declaration
 #ifndef VL_SIG
+# define VL_SIG8(name, msb,lsb)		uint8_t  name
 # define VL_SIG(name, msb,lsb)		uint32_t name
 # define VL_SIGW(name, msb,lsb, words)	uint32_t name[words]
+# define VL_IN8(name, msb,lsb)		uint8_t  name
 # define VL_IN(name, msb,lsb)		uint32_t name
 # define VL_INW(name, msb,lsb, words)	uint32_t name[words]
+# define VL_INOUT8(name, msb,lsb)	uint8_t  name
 # define VL_INOUT(name, msb,lsb)	uint32_t name
 # define VL_INOUTW(name, msb,lsb, words) uint32_t name[words]
+# define VL_OUT8(name, msb,lsb)		uint8_t  name
 # define VL_OUT(name, msb,lsb)		uint32_t name
 # define VL_OUTW(name, msb,lsb, words)	uint32_t name[words]
 # define VL_PORT(name, msb,lsb)		uint32_t name		// Depreciated
